
#pragma once

// Genode includes
#include <base/rpc.h>
#include <dataspace/capability.h>
#include <session/session.h>


namespace Verify_signature
{
	using Genode::Dataspace_capability;
	using Genode::size_t;
	using Genode::uint8_t;

	enum class Result {
		INVALID_CERTIFICATE,
		INVALID_SIGNATURE,
		SUCCESS
	};

	struct Session;

	class Buffer_too_small : public Genode::Exception { };
	class Hsm_error        : public Genode::Exception { };
}


struct Verify_signature::Session : Genode::Session
{
	/**
	 * \noapi
	 */
	static const char *service_name() { return "Verify_signature"; }

	/*
	 * A Verify_signature session consumes a dataspace capability for the server's
	 * session-object allocation, its session capability, and two dataspace
	 * capabilities for the communication buffers.
	 */
	enum { CAP_QUOTA = 4 };

	/**
	 * Check if the signature of a request is correct.
	 *
	 * @param  data               byte pointer to the data (message) buffer
	 * @param  data_size          data (message) size in bytes
	 * @param  certificate        byte pointer to the certificate buffer
	 * @param  certificate_size   certificate size in bytes
	 * @param  signature          byte pointer to the signature buffer
	 * @param  signature_size     signature size in bytes
	 */
	virtual Result verify(const uint8_t* data,        const size_t data_size,
	                      const uint8_t* certificate, const size_t certificate_size,
	                      const uint8_t* signature,   const size_t signature_size) = 0;

	/*******************
	 ** RPC interface **
	 *******************/

	GENODE_RPC_THROW(Rpc_verify, Result, _verify, GENODE_TYPE_LIST(Buffer_too_small, Hsm_error), size_t, size_t, size_t);

	GENODE_RPC(Rpc_dataspace_data,        Dataspace_capability, _dataspace_data);
	GENODE_RPC(Rpc_dataspace_certificate, Dataspace_capability, _dataspace_certificate);
	GENODE_RPC(Rpc_dataspace_signature,   Dataspace_capability, _dataspace_signature);

	GENODE_RPC_INTERFACE(Rpc_verify, Rpc_dataspace_data, Rpc_dataspace_certificate, Rpc_dataspace_signature);
};
